"""
Custom settings for QGIS plugin
Denis Rouzaud
denis.rouzaud@gmail.com
Feb. 2013
"""

from PyQt4.QtGui import QDialog


class PluginSettings():     
    def __init__(self, pluginName, settings, setValuesOnDialogAccepted=True, setValueOnWidgetUpdate=False):
        self.pluginName = pluginName
        self.settings = []
        self.setValueOnWidgetUpdate = setValueOnWidgetUpdate
        if isinstance(self, QDialog) and setValuesOnDialogAccepted:
            self.accepted.connect(self.acceptDialog)
        for setting in settings:
            self.addSetting(setting)

    def acceptDialog(self):
        if self.onBeforeAcceptDialog():
            self.setValuesFromWidgets()

    '''
    you can override this method in the PluginSettings subclass
    '''
    def onBeforeAcceptDialog(self):
        return True

    def addSetting(self, setting):
        if self.setting(setting.name) is not None:
            raise NameError("%s already exist in settings." % setting.name)
        if hasattr(self, setting.name):
            setting.setWidget(getattr(self, setting.name))
            if self.setValueOnWidgetUpdate:
                setting.connectGuiSignal()
        self.settings.append(setting)
          
    def showEvent(self, e):
        self.setWidgetsFromValues()

    def setting(self, name):
        for setting in self.settings:
            if setting.name == name:
                return setting
        return None

    def value(self, settingName):
        setting = self.setting(settingName)
        if setting is None:
            raise NameError('%s has no setting %s' % (self.pluginName, settingName))
        return setting.getValue()

    def setValue(self, settingName, value):
        setting = self.setting(settingName)
        if setting is None:
            raise NameError('%s has no setting %s' % (self.pluginName, settingName))
        setting.setValue(value)
          
    def setValuesFromWidgets(self):
        for setting in self.settings:
            if setting.widget is not None:
                setting.setValueFromWidget()

    def setWidgetsFromValues(self):
        for setting in self.settings:
            if setting.widget is not None:
                setting.setWidgetFromValue()