"""
Custom settings for QGIS plugin
Denis Rouzaud
denis.rouzaud@gmail.com
Feb. 2013
"""
# options:
# comboMode: can be data or text. It defines if setting is found directly in combobox text or rather in the userData.
        
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *


class String(QObject):
    valueChanged = pyqtSignal(str)
        
    def __init__(self, pluginName, name, scope, defaultValue, options={}):
        QObject.__init__(self)
        if scope not in ("global", "project"):
            raise NameError("%s is not a valid scope. Must be project or global." % scope)
        self.pluginName = pluginName
        self.name = name
        self.scope = scope
        self.defaultValue = defaultValue
        self.widget = None
        self.options = options
        self.check(defaultValue)
        
    def setWidget(self, widget):
        self.widget = widget
        
    def check(self, value):
        if type(value) != str and type(value) != QString:
            raise NameError("Setting %s must be a string." % self.name)

    def setValue(self, value):
        self.check(value)
        if self.scope == "global":
            QSettings(self.pluginName, self.pluginName).setValue(self.name, value)
        elif self.scope == "project":
            QgsProject.instance().writeEntry(self.pluginName, self.name, value)
        self.valueChanged.emit(value)

    def getValue(self):
        if self.scope == "global":
            return QSettings(self.pluginName,self.pluginName).value(self.name, self.defaultValue).toString()
        elif self.scope == "project":
            return QgsProject.instance().readEntry(self.pluginName, self.name, self.defaultValue)[0]
        
    def connectGuiSignal(self):
        if self.widget is None:
            return
        if type(self.widget) == QLineEdit:
            signal = SIGNAL("textChanged(QString)")
        elif type(self.widget) == QButtonGroup:
            signal = SIGNAL("buttonClicked(int)")
        elif type(self.widget) == QComboBox:
            signal = SIGNAL("activated(int)")
        else:
            print "Error: ", self.name, type(self.widget)
            raise NameError("PluginSettings does not handle %s widgets for strings for the moment" % type(self.widget))
        QObject.connect(self.widget, signal, self.setValueFromWidget)
        
    def setWidgetFromValue(self):
        if self.widget is None:
            return
        text = self.getValue()
        if type(self.widget) == QLineEdit:
            self.widget.setText(text)
        elif type(self.widget) == QButtonGroup:
            # for checkboxes
            for button in self.widget.buttons():
                if text == button.objectName():
                    button.setChecked(True)
                    break
        elif type(self.widget) == QComboBox:
            idx = -1
            comboMode = self.options.get("comboMode", "data")
            if comboMode == 'data':
                idx = self.widget.findData(text)
            elif comboMode == 'text':
                idx = self.widget.findText(text)
            if idx != -1:
                self.widget.setCurrentIndex(idx)
        else:
            raise NameError("PluginSettings does not handle %s widgets for strings for the moment" % type(self.widget))

    def setValueFromWidget(self, i=0):
        value = None
        if type(self.widget) == QLineEdit:
            value = self.widget.text()
        elif type(self.widget) == QButtonGroup:
            for button in self.widget.buttons():
                if button.isChecked():
                    value = button.objectName()
            if value is None:
                raise NameError("No widget was checked for setting %s" % self.name)
        elif type(self.widget) == QComboBox:
            comboMode = self.options.get("comboMode", "data")
            if comboMode == "data":
                idx = self.widget.currentIndex()
                value = self.widget.itemData(idx).toString()
            elif comboMode == "text":
                value = self.widget.currentText()
        else:
            raise NameError("PluginSettings does not handle %s widgets for strings for the moment" % type(self.widget))
        self.setValue(value)
